<p>在本文中让我们来看看在 Java 8 中引入的 2 个新的类： <em>Period</em> 和 <em>Duration</em>.</p> <br><p>上面 2 个类可以被用来替换在 determine 和 time 中大量使用用来计算 2 个时间不同的 API。</p> <br><p>针对上面 2 个类最主要的不同就是 <em>Period</em> 被用来计算日期的不同，<em>Duration</em> 则是被用来计算时间的不同。</p> <br><h3>Period 类</h3> <br><p><em>Period</em> 使用的单位是 年，月，日来表达 2 个日期之间的不同。</p> <br><p>我们可以通过 2 个时期之间不同的 <em>between()</em> 方法来获得 <em>Period</em> 对象。</p> <br><p></p> <br><pre><code>LocalDate startDate = LocalDate.of(2015, 2, 20);<br>LocalDate endDate = LocalDate.of(2017, 1, 15);<br><br>Period period = Period.between(startDate, endDate);<br></code></pre> <br><p>当我们获得 Period 对象后，我们可以用 Period 对象中的 *getYears(), getMonths(), getDays() 方法来获得具体的值。</p> <br><p></p> <br><pre><code>        logger.info(String.format("Years:%d months:%d days:%d", period.getYears(), period.getMonths(), period.getDays()));<br></code></pre> <br><p></p> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/3X/8/1/81785b27f4b88207f064c1ce65dbd8cb9e7425bd.jpeg" rel="nofollow"><img alt="2025-01-17_06-47-08" height="373" src="https://i-blog.csdnimg.cn/img_convert/c5f99125a0a909c34ca9162be93c23a8.jpeg" width="690" /></a></p> <br><p></p> <br><p><em>isNegative()</em> 也是 Period 中的一个方法，可以通过这个方法来判断 2 个比较时间的大小关系。</p> <br><p>当这个方法返回 True 的时候，表明结束时间要大于起始时间。</p> <br><p></p> <br><pre><code>assertFalse(period.isNegative());<br></code></pre> <br><p>相反的 <em>isNegative()</em> 如果返回的为 false，那么表明 startDate* 要早入 <em>endDate</em> 的值。</p> <br><p>另外一个可以创建 <em>Period</em> 对象的方法就是使用数字，Period 中提供了一个 of 方法，我们可以用这个方法来构造一个 Period 对象：</p> <br><p></p> <br><pre><code>Period fromUnits = Period.of(3, 10, 10);<br>Period fromDays = Period.ofDays(50);<br>Period fromMonths = Period.ofMonths(5);<br>Period fromYears = Period.ofYears(10);<br>Period fromWeeks = Period.ofWeeks(40);<br><br>assertEquals(280, fromWeeks.getDays());<br></code></pre> <br><p>如果 of 方法中只有一个输入参数的话，那么默认调用的是 <em>ofDays()</em> 方法，等于用输入的数字初始化日这个单位。,</p> <br><p>基于上面的设计， <em>ofWeeks()</em> 方法使用的是乘以 7 的方式，因为不管一年有多少天，但一周 7 天这个肯定是不会变的。</p> <br><p>通过看 JDK 的源代码：</p> <br><p></p> <br><pre><code>    public static Period ofWeeks(int weeks) {<br>        return create(0, 0, Math.multiplyExact(weeks, 7));<br>    }<br></code></pre> <br><p></p> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/3X/f/f/ff3b2b424c41fd2cb9a1931e28810d6e0657a0e2.jpeg" rel="nofollow"><img alt="2025-01-17_06-53-42" height="500" src="https://i-blog.csdnimg.cn/img_convert/e9636754fbbd0df29e306e4e1afa9876.jpeg" width="530" /></a></p> <br><p></p> <br><p>上面的构造方法就能说明这个问题了。</p> <br><p>同时，还可以使用字符串来创建 <em>Period</em> 对象，这个字符串使用的序列为 “PnYnMnD”：</p> <br><p></p> <br><pre><code>Period fromCharYears = Period.parse("P2Y");<br>assertEquals(2, fromCharYears.getYears());<br><br>Period fromCharUnits = Period.parse("P2Y3M5D");<br>assertEquals(5, fromCharUnits.getDays());<br></code></pre> <br><p>当 period 对象创建成功后，可以使用 <em>plusX()</em> 和 <em>minusX()</em> 方法来对创建的 period 对象中的值进行调整：</p> <br><p></p> <br><pre><code>assertEquals(56, period.plusDays(50).getDays());<br>assertEquals(9, period.minusMonths(2).getMonths());<br></code></pre> <br><h3>Duration 类</h3> <br><p>和 Period 相对，Duration 类是对时间进行操作的。</p> <br><p>具体操作的单位为秒（seconds ）和纳秒（nanoseconds ）。</p> <br><p>因可以直接对纳秒进行操作，所以 Duration 能比较精确的对时间进行计算。</p> <br><p>另外，要获得 Duration 对象，我们需要从 instants 来进行比较，使用 between() 方法来比较 instants 。</p> <br><p></p> <br><pre><code>Instant start = Instant.parse("2017-10-03T10:15:30.00Z");<br>Instant end = Instant.parse("2017-10-03T10:16:30.00Z");<br>        <br>Duration duration = Duration.between(start, end);<br></code></pre> <br><p>上面的方法能够构造一个 Duration 对象，然后从 Duration 对象中使用 getSeconds() 或者getNanoseconds() 方法来获得具体的单位值：</p> <br><p></p> <br><pre><code>assertEquals(60, duration.getSeconds());<br></code></pre> <br><p>可选的，因为是对时间进行计算机，那么你还可以从 LocalDateTime 实例中来比较构造一个 Duration 对象：</p> <br><p></p> <br><pre><code>LocalDateTime start = LocalDateTime.parse("2020-01-01T08:00:00");<br>LocalDateTime end = LocalDateTime.parse("2023-01-01T12:00:00");<br><br>Duration.between(start, end).getSeconds();<br></code></pre> <br><p>基于起始时间的不同，Duration 可能会有下面的 3 中情况：</p> <br><ul><li>Negative: start < end</li><li>Zero: start = end</li><li>Positive: start > end</li></ul> <br><p>同时，标准的时间 Time API 也提供了转发方法来检查 Duration 是否为 0 或者负数：</p> <br><p></p> <br><pre><code>LocalDateTime start = LocalDateTime.parse("2020-01-01T08:00:00");<br>LocalDateTime end = LocalDateTime.parse("2020-01-01T12:00:00");<br>assertFalse(Duration.between(start, end).isNegative());<br>assertTrue(Duration.between(end, start).isNegative());<br><br>LocalDateTime theTime = LocalDateTime.parse("2023-09-09T08:00:00");<br>assertTrue(Duration.between(theTime,theTime).isZero());<br>assertFalse(Duration.between(theTime,theTime).isNegative());<br></code></pre> <br><p></p> <br><pre><code>// Java version 18+ required<br>LocalDateTime start = LocalDateTime.parse("2020-01-01T08:00:00");<br>LocalDateTime end = LocalDateTime.parse("2020-01-01T12:00:00");<br>assertTrue(Duration.between(start, end).isPositive());<br>assertFalse(Duration.between(end, start).isPositive());<br></code></pre> <br><p>当然，我们也可以使用单位来构造一个 Duration 对象，这个和 Period 的构造方法基本相同。</p> <br><p>可以使用的方法有： <em>ofDays(), ofHours(), ofMillis(), ofMinutes(), ofNanos(), ofSeconds()</em>:</p> <br><p></p> <br><pre><code>Duration fromDays = Duration.ofDays(1);<br>assertEquals(86400, fromDays.getSeconds());<br>       <br>Duration fromMinutes = Duration.ofMinutes(60);<br></code></pre> <br><p>我们还可以使用字符串的方法来构造 <em>Duration</em> ，表达方式为：<em>PnDTnHnMn.nS</em>：</p> <br><p></p> <br><pre><code>Duration fromChar1 = Duration.parse("P1DT1H10M10.5S");<br>Duration fromChar2 = Duration.parse("PT10M");<br></code></pre> <br><p>duration 对象也可以实现其他单位的转换，使用的方法为：<em>toDays(), toHours(), toMillis(), toMinutes()</em>：</p> <br><p></p> <br><pre><code>assertEquals(1, fromMinutes.toHours());<br></code></pre> <br><p>针对时间的增加和减少，我们可以使用 <em>plusX()</em> 方法 <em>minusX()</em>, <em>X</em> 根据具体的单位不同而使用不同的字符来表达，具体的可以参考 JDK API 的手册。</p> <br><p>下面我们只简单的列了几个可以使用的方法：</p> <br><p></p> <br><pre><code>assertEquals(120, duration.plusSeconds(60).getSeconds());     <br>assertEquals(30, duration.minusSeconds(30).getSeconds());<br></code></pre> <br><p>为了避免书写上面的问题，我们还可以在 <em>plus()</em> 和 <em>minus()</em> 方法中使用 <em>ChronoUnit</em> 来制定添加或者减少的单位：</p> <br><p></p> <br><pre><code>assertEquals(120, duration.plus(60, ChronoUnit.SECONDS).getSeconds());     <br>assertEquals(30, duration.minus(30, ChronoUnit.SECONDS).getSeconds());<br></code></pre> <br><h3>结论</h3> <br><p>让我为你总结一下这两个类的主要区别和用法：</p> <br><h4>Period 和 Duration 的主要区别</h4> <br><ol><li><strong>Period</strong> <br>  <ul><li>使用基于日期的值（date-based values）</li><li>主要用于处理年、月、日的时间段</li><li>适用于较长时间段的计算</li></ul></li><li><strong>Duration</strong> <br>  <ul><li>使用基于时间的值（time-based values）</li><li>主要用于处理秒和纳秒级别的时间</li><li>适用于更短、更精确的时间间隔</li></ul></li></ol> <br><h4>Period 的主要用法</h4> <br><ol><li>创建 Period 对象：</li></ol> <br><p></p> <br><pre><code>// 通过两个日期之间的差值创建<br>LocalDate startDate = LocalDate.of(2015, 2, 20);<br>LocalDate endDate = LocalDate.of(2017, 1, 15);<br>Period period = Period.between(startDate, endDate);<br><br>// 直接创建特定时间段<br>Period fromUnits = Period.of(3, 10, 10);  // 年、月、日<br>Period fromDays = Period.ofDays(50);<br>Period fromMonths = Period.ofMonths(5);<br>Period fromYears = Period.ofYears(10);<br>Period fromWeeks = Period.ofWeeks(40);<br></code></pre> <br><ol><li>解析字符串创建 Period：</li></ol> <br><p></p> <br><pre><code>Period fromCharYears = Period.parse("P2Y");        // 2年<br>Period fromCharUnits = Period.parse("P2Y3M5D");    // 2年3月5日<br></code></pre> <br><h4>Duration 的主要用法</h4> <br><ol><li>创建 Duration 对象：</li></ol> <br><p></p> <br><pre><code>// 通过两个时间点之间的差值创建<br>Instant start = Instant.parse("2017-10-03T10:15:30.00Z");<br>Instant end = Instant.parse("2017-10-03T10:16:30.00Z");<br>Duration duration = Duration.between(start, end);<br><br>// 直接创建特定时长<br>Duration fromDays = Duration.ofDays(1);<br>Duration fromMinutes = Duration.ofMinutes(60);<br></code></pre> <br><ol><li>Duration 的其他操作：</li></ol> <br><p></p> <br><pre><code>// 检查 Duration 的性质<br>duration.isNegative();<br>duration.isZero();<br>duration.isPositive();  // Java 18+<br><br>// 转换到其他时间单位<br>duration.toDays();<br>duration.toHours();<br>duration.toMillis();<br>duration.toMinutes();<br><br>// 增加或减少时长<br>duration.plusSeconds(60);<br>duration.minusSeconds(30);<br></code></pre> <br><h4>使用建议</h4> <br><ol><li>如果你需要处理年、月、日级别的时间段，使用 <strong>Period</strong></li><li>如果你需要更精确的时间计算（包括小时、分钟、秒、纳秒），使用 <strong>Duration</strong></li><li>Period 可能会损失精度，而 Duration 则保持高精度</li></ol> <br><p>这两个类都是在 Java 8 中引入的，属于新的日期时间 API 的一部分，它们提供了更清晰和更易用的时间间隔处理方式。</p> <br><p></p> <br><p><a href="https://www.isharkfly.com/t/java-period-duration/16809" rel="nofollow" title="Java 中的 Period 和 Duration - Java - iSharkFly">Java 中的 Period 和 Duration - Java - iSharkFly</a></p>