<p><code>innodb_buffer_pool_size</code>是MySQL InnoDB存储引擎最重要的参数之一，它直接影响数据库的性能。这个参数决定了InnoDB缓存表数据和索引数据的内存大小。</p> <br><h3>基本配置原则</h3> <br><ol><li><strong>一般建议</strong>：将<code>innodb_buffer_pool_size</code>设置为服务器物理内存的50%-80%</li><li><strong>小型系统</strong>：如果是小型系统或MySQL与其他服务共享服务器，可以设置为25%-50%</li><li><strong>专用数据库服务器</strong>：如果服务器专门用于MySQL，可以设置为物理内存的80%左右</li></ol> <br><h3>具体配置方法</h3> <br><p>在my.cnf或my.ini文件中设置：</p> <br><p></p> <br><pre><code>[mysqld]<br>innodb_buffer_pool_size = 8G  # 例如设置为8GB<br></code></pre> <br><h3>注意事项</h3> <br><ol><li><strong>避免过大</strong>：不要将此值设置得太大，否则可能导致操作系统开始使用交换空间，严重降低性能</li><li><strong>在线调整</strong>：MySQL 5.7及以上版本支持动态调整此参数，无需重启服务器： <p></p> <pre><code>SET GLOBAL innodb_buffer_pool_size = 8589934592;  # 设置为8GB (以字节为单位)<br></code></pre> </li><li><strong>多个缓冲池实例</strong>：对于大内存服务器，建议配置多个缓冲池实例以提高并发性能： <p></p> <pre><code>innodb_buffer_pool_instances = 8  # 例如设置为8个实例<br></code></pre> </li><li><strong>监控使用情况</strong>：通过以下命令监控缓冲池的使用情况： <p></p> <pre><code>SHOW ENGINE INNODB STATUS\G<br>SELECT * FROM information_schema.INNODB_BUFFER_POOL_STATS;<br></code></pre> </li><li><strong>预热功能</strong>：考虑使用InnoDB缓冲池预热功能，特别是对于频繁重启的服务器： <p></p> <pre><code>innodb_buffer_pool_dump_at_shutdown = 1<br>innodb_buffer_pool_load_at_startup = 1<br></code></pre> </li></ol> <br><p>最佳的设置取决于您的工作负载特性、服务器配置和其他运行在同一系统上的应用程序。建议在调整后监控系统性能，并根据实际情况进行微调。</p> <br><h3>MariaDB</h3> <br><p>针对 MairaDB，修改配置文件路径为： <code>nano /etc/mysql/mariadb.conf.d/50-server.cnf</code></p> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/3X/5/d/5d0cf2cdd4fa1e08566054a6164e60567f2991b9.jpeg" rel="nofollow"><img alt="2025-03-13_14-27-17" height="403" src="https://i-blog.csdnimg.cn/img_convert/3846ce118056dd4186a5f82f68688699.jpeg" width="690" /></a></p> <br><p>针对我们的服务器，我们的的这个配置被修改成了 24G。</p> <br><p>查询 Pool 大小：</p> <br><p><code>SELECT * FROM information_schema.INNODB_BUFFER_POOL_STATS;</code></p> <br><h3>INNODB_BUFFER_POOL_STATS</h3> <br><p></p> <br><table><thead><tr><th>POOL_ID</th><th>POOL_SIZE</th><th>FREE_BUFFERS</th><th>DATABASE_PAGES</th><th>OLD_DATABASE_PAGES</th><th>MODIFIED_DATABASE_PAGES</th><th>PENDING_DECOMPRESS</th><th>PENDING_READS</th><th>PENDING_FLUSH_LRU</th><th>PENDING_FLUSH_LIST</th><th>PAGES_MADE_YOUNG</th><th>PAGES_NOT_MADE_YOUNG</th><th>PAGES_MADE_YOUNG_RATE</th><th>PAGES_MADE_NOT_YOUNG_RATE</th><th>NUMBER_PAGES_READ</th><th>NUMBER_PAGES_CREATED</th><th>NUMBER_PAGES_WRITTEN</th><th>PAGES_READ_RATE</th><th>PAGES_CREATE_RATE</th><th>PAGES_WRITTEN_RATE</th><th>NUMBER_PAGES_GET</th><th>HIT_RATE</th><th>YOUNG_MAKE_PER_THOUSAND_GETS</th><th>NOT_YOUNG_MAKE_PER_THOUSAND_GETS</th><th>NUMBER_PAGES_READ_AHEAD</th><th>NUMBER_READ_AHEAD_EVICTED</th><th>READ_AHEAD_RATE</th><th>READ_AHEAD_EVICTED_RATE</th><th>LRU_IO_TOTAL</th><th>LRU_IO_CURRENT</th><th>UNCOMPRESS_TOTAL</th><th>UNCOMPRESS_CURRENT</th></tr></thead><tbody><tr><td>0</td><td>1557632</td><td>1486272</td><td>70609</td><td>26044</td><td>2601</td><td>1504</td><td>0</td><td>0</td><td>0</td><td>47084</td><td>511720</td><td>0</td><td>0</td><td>73858</td><td>4808</td><td>3023</td><td>0</td><td>0</td><td>0</td><td>10046433153</td><td>1000</td><td>0</td><td>0</td><td>6422</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td><td>0</td></tr></tbody></table> <br><p>上面的配置参数为生产服务器上的真实参数。</p> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/3X/f/2/f20d6ca8d9e9fd76109e84861de0de22400129f9.jpeg" rel="nofollow"><img alt="2025-03-13_14-45-02" height="95" src="https://i-blog.csdnimg.cn/img_convert/7f3d2a93711d24a72d8ef3ad932a9e8f.jpeg" width="690" /></a></p> <br><p></p> <br><h3>引擎状态查询</h3> <br><p><code>SHOW ENGINE INNODB STATUS</code></p> <br><p>上面的命令能够查询当前引擎的使用状态：</p> <br><p></p> <br><pre><code><br>=====================================<br>2025-03-13 18:34:22 0x75a2fc5606c0 INNODB MONITOR OUTPUT<br>=====================================<br>Per second averages calculated from the last 42 seconds<br>-----------------<br>BACKGROUND THREAD<br>-----------------<br>srv_master_thread loops: 10 srv_active, 0 srv_shutdown, 196110 srv_idle<br>srv_master_thread log flush and writes: 196115<br>----------<br>SEMAPHORES<br>----------<br>------------<br>TRANSACTIONS<br>------------<br>Trx id counter 1235652<br>Purge done for trx's n:o < 1235652 undo n:o < 0 state: running but idle<br>History list length 0<br>LIST OF TRANSACTIONS FOR EACH SESSION:<br>---TRANSACTION (0x75a940405880), not started<br>mysql tables in use 1, locked 0<br>0 lock struct(s), heap size 1128, 0 row lock(s)<br>---TRANSACTION (0x75a940404d80), ACTIVE 0 sec fetching rows<br>mysql tables in use 1, locked 0<br>0 lock struct(s), heap size 1128, 0 row lock(s)<br>MariaDB thread id 5221894, OS thread handle 129343170938560, query id 399272277 ns564012.ip-54-39-157.net 54.39.157.60 src Sending data<br>SELECT count(*) FROM `release` WHERE repo_id=? AND sha1<>?<br>Trx read view will not see trx with id >= 1235652, sees < 1235652<br>---TRANSACTION (0x75a940401680), not started<br>0 lock struct(s), heap size 1128, 0 row lock(s)<br>--------<br>FILE I/O<br>--------<br>Pending flushes (fsync): 0<br>74899 OS file reads, 93031 OS file writes, 90571 OS fsyncs<br>0.00 reads/s, 0 avg bytes/read, 0.36 writes/s, 0.36 fsyncs/s<br>---<br>LOG<br>---<br>Log sequence number 2828162553<br>Log flushed up to   2828162553<br>Pages flushed up to 2765805507<br>Last checkpoint at  2765805507<br>----------------------<br>BUFFER POOL AND MEMORY<br>----------------------<br>Total large memory allocated 25803358208<br>Dictionary memory allocated 155238072<br>Buffer pool size   1557632<br>Free buffers       1486272<br>Database pages     70609<br>Old database pages 26044<br>Modified db pages  2601<br>Percent of dirty pages(LRU & free pages): 0.167<br>Max dirty pages percent: 90.000<br>Pending reads 0<br>Pending writes: LRU 0, flush list 0<br>Pages made young 47083, not young 511720<br>0.00 youngs/s, 0.00 non-youngs/s<br>Pages read 73858, created 4808, written 3023<br>0.00 reads/s, 0.00 creates/s, 0.00 writes/s<br>Buffer pool hit rate 1000 / 1000, young-making rate 0 / 1000 not 0 / 1000<br>Pages read ahead 0.00/s, evicted without access 0.00/s, Random read ahead 0.00/s<br>LRU len: 70609, unzip_LRU len: 1504<br>I/O sum[0]:cur[0], unzip sum[0]:cur[0]<br>--------------<br>ROW OPERATIONS<br>--------------<br>1 read views open inside InnoDB<br>state: sleeping<br>----------------------------<br>END OF INNODB MONITOR OUTPUT<br>============================<br><br></code></pre> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/3X/2/a/2acccdfc3db2ff51b19af5500d03bfd5aedfe8c3.jpeg" rel="nofollow"><img alt="2025-03-13_14-46-05" height="500" src="https://i-blog.csdnimg.cn/img_convert/c81ceb5d82c93c708e62646ee6ec22fa.jpeg" width="686" /></a></p> <br><p></p> <br><p>数据库的参数，通常可以用查询来了解。</p> <br><p></p> <br><p><a href="https://www.isharkfly.com/t/mysql-innodb-buffer-pool-size/17058/2" rel="nofollow" title="MySQL的innodb_buffer_pool_size参数优化配置指南 - #2 by hex - 数据库 - iSharkFly">MySQL的innodb_buffer_pool_size参数优化配置指南 - #2 by hex - 数据库 - iSharkFly</a></p>