<p>python stomp activemq mess send message in loop</p> <br><p></p> <br><p>To send messages to an ActiveMQ broker using the STOMP protocol in a loop with Python, you can use the <code>stomp.py</code> library. First, you’ll need to ensure the library is installed. You can install it using pip if you haven’t already:</p> <br><p></p> <br><pre><code>pip install stomp.py<br></code></pre> <br><p>Here’s a simple example of how to send messages in a loop:</p> <br><p></p> <br><pre><code>import stomp<br>import time<br><br>class MyListener(stomp.ConnectionListener):<br>    def on_error(self, headers, message):<br>        print('received an error: "%s"' % message)<br>    def on_disconnected(self):<br>        print('disconnected')<br><br>def send_messages():<br>    conn = stomp.Connection([('localhost', 61613)])<br>    conn.set_listener('', MyListener())<br>    conn.connect('admin', 'password', wait=True)<br><br>    for i in range(10):  # Send 10 messages<br>        message = f'Message number {i + 1}'<br>        conn.send(destination='/queue/test', body=message)<br>        print(f'Sent: {message}')<br>        time.sleep(1)<br><br>    conn.disconnect()<br><br>if __name__ == '__main__':<br>    send_messages()<br></code></pre> <br><h4>Key Points:</h4> <br><ol><li><strong>Connection Setup</strong>: Replace <code>'localhost'</code> and <code>61613</code> with your ActiveMQ server’s host and port.</li><li><strong>Authentication</strong>: Replace <code>'admin'</code> and <code>'password'</code> with your ActiveMQ username and password.</li><li><strong>Queue Name</strong>: Adjust the <code>destination='/queue/test'</code> to the queue you want to send messages to.</li><li><strong>Loop</strong>: Adjust the range and <code>time.sleep()</code> for the number of messages and delay between sends as needed.</li></ol> <br><p>This script establishes a connection to the ActiveMQ broker, sends a series of messages to a specified queue, and then disconnects.</p> <br><h3>GPT</h3> <br><p>GPT 的回答基本上解决了问题。</p> <br><p>遇到的问题是，在向 ActiveMQ 发送消息的时候，我们有一个 SendMQ 的方法。</p> <br><p>在这个方法将会打开连接发送消息后关闭连接。</p> <br><p>我们的问题在：现在需要向 MQ 发送 1 万多条消息，这个循环放那里。</p> <br><p>开始我们把这个循环放在了内层，这里就出现了一个问题，Python 的循环会异步调用 Close 这方法，然后导致整个程序的挂起。</p> <br><p>通过上面 GPT 的回答，我知道应该在整个循环的最外层打开连接，循环内不能再次打开连接和关闭连接。</p> <br><p>等把这 1 万多条消息发送成功后再关闭连接。</p> <br><p>同样的问题，我们也使用了 claude_3_haiku_bot 这个模型来问了这个问题，请参考后面的消息。</p> <br><p>感觉 GPT4 的模型对这个问题回答得更好一些。</p> <br><p>Let’s see what topics on “python stomp sending in loop” I can find on the iSharkFly forum:</p> <br><p>Search</p> <br><p>Hmm, I couldn’t find any specific topics about “python stomp sending in loop” on this forum. Let me try a more general search:</p> <br><p>Search</p> <br><p>The search results provide some relevant information about using the Python <code>stomp</code> library to send messages, including:</p> <br><ul><li>There was an issue where the messages were being sent as binary data instead of text, due to the <code>auto_content_length</code> setting. Setting this to <code>False</code> resolved the issue.</li><li>There is sample code showing how to connect to an ActiveMQ message queue using <code>stomp</code>, send a message, and disconnect.</li></ul> <br><p>Based on this, it seems the main issue you are encountering is getting the messages to display correctly on the message server. The key things to try are:</p> <br><ol><li>Make sure you are setting <code>auto_content_length=False</code> when creating the <code>stomp</code> connection.</li><li>Ensure the message content you are sending is properly formatted as text, not binary data.</li><li>Check that the message server is configured to display text messages correctly.</li></ol> <br><p>If you are still having issues, you could try searching for more specific information about the “python stomp sending in loop” use case, or providing more details about the exact problem you are facing.</p> <br><p></p> <br><p>Google AI 的回复貌似也还行。</p> <br><p></p> <br><p></p> <br><p class="img-center"><a href="https://cdn.isharkfly.com/com-isharkfly-www/discourse-uploads/original/2X/a/a70f68246fb9df667efb5fdb88d2b97980bed54c.png" rel="nofollow"><img alt="2024-10-12_21-38-40" height="500" src="https://img-blog.csdnimg.cn/img_convert/c3147497e5643ffe126b75c133f139e1.png" width="630" /></a></p> <br><p></p> <br><p>关于写代码，真的可以问问 AI。</p> <br><p></p> <br><p><a href="https://www.isharkfly.com/t/python-stomp-activemq/16420" rel="nofollow" title="https://www.isharkfly.com/t/python-stomp-activemq/16420">https://www.isharkfly.com/t/python-stomp-activemq/16420</a></p>